/*****************************************************************************/
/**
 * \file main.h
 * \author Copyright (c) 2024, Murray R. Van Luyn. <green@bug-eyed.monster>
 * \version 0.2
 * \date 26-04-24
 * Instructions: https://green.bug-eyed.monster/arduino-i2c-slave/
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#include <Arduino.h>
#include <Wire.h>

// List headers of modules that main references code in here
#include "myservo.h"
#include "myadc.h"


/*****************************************************************************
 *
 *                           Constant Definitions
 *
 *****************************************************************************/
// Comment out to remove debug reporting macros
//#define I2C_DEBUG

// Define I2C address for your slave device
#define I2C_ADDRESS 0x08

// Define number of 16-bit registers for your slave device
#define I2C_NUM_REGS 8

// Define the labels for slave device 16-bit registers
#define I2C_REG_0 0    // R/W - Assigned to myservo module
#define I2C_REG_1 1    // RO - Assigned to myadc module
#define I2C_REG_2 2
#define I2C_REG_3 3
#define I2C_REG_4 4
#define I2C_REG_5 5
#define I2C_REG_6 6
#define I2C_REG_7 7


/*****************************************************************************
 *
 *                       Atomic Register Access Macros
 *
 *****************************************************************************/

// Apply these to I2C registers operations, undertaken outside I2C service
// ISR context, to ensure effective atomicity in access.

#define I2C_ATOMIC_REG_WR(y, x)     cli(); /* Disable interrupts, prevent */ \ 
                                           /* half-written register read, */ \
                                           /* within I2C requestEvent ISR.*/ \
                                    y = x;                                   \
                                    sei();                                   


#define I2C_ATOMIC_REG_RD(x, y)     cli(); /* Disable interrupts, prevent */ \
                                           /* receiveEvent ISR writing    */ \
                                           /* I2C register, during a read */ \
                                           /* in the task loop.           */ \
                                    x = y;                                   \
                                    sei();                                   


/*****************************************************************************
 *
 *                                Debug Macros
 *
 *****************************************************************************/
#ifdef I2C_DEBUG
  #define I2C_DEBUG_SETUP()             Serial.begin(9600);              \
                                        Serial.print("Setup, ready!\n"); \

  #define I2C_WRITE_DEBUG(reg, val)     Serial.print("Wrote "); \
                                        Serial.print(val, DEC); \
                                        Serial.print(" to ");   \
                                        Serial.print(reg, DEC); \
                                        Serial.print("\n");

  #define I2C_READ_DEBUG(reg, val)      Serial.print("Read ");  \
                                        Serial.print(val, DEC); \
                                        Serial.print(" from "); \
                                        Serial.print(reg, DEC); \
                                        Serial.print("\n");   
#else 
  #define I2C_DEBUG_SETUP()

  #define I2C_WRITE_DEBUG(reg, val)

  #define I2C_READ_DEBUG(reg, val)
#endif             
